@@ -29,7 +29,8 @@ import {
     ShowErrorPopup,
     StackItem,
     TemplatesScriptsManager,
-    WindowManager
+    WindowManager,
+    WidgetManager
 } from '../dependencies/localFiles.js';
 
 import {Adw, Gtk, Gdk, Gio, GLib, GLibUnix} from '../dependencies/gi.js';
@@ -81,13 +82,14 @@ const DesktopManager = class {
         this.desktopMenuManager = new DesktopMenu.DesktopBackgroundMenu(this);
         this.Prefs.init(this);
         this.shortcutManager = new ShortcutManager(this);
+        this.widgetManager = new WidgetManager.WidgetManager(this);
 
         // Init Variables
         this._clickX = null;
         this._clickY = null;
         this._compositeStackList = null;
         this._displayList = [];
-        this.ignoreKeys = this.Enums.IgnoreKeys.map(_k => Gdk._k);
+        this.ignoreKeys = this.Enums.IgnoreKeys.map(_k => Gdk[_k]);
 
         // setup gracefull termination
         if (this._asDesktop) {
@@ -145,6 +147,8 @@ const DesktopManager = class {
         // desktopdir, even if a prior fileList was read, the
         // forced new read will recalculate and resave new
         // normalized coordinates and monitor information.
+
+        this._startWidgetDisplay();
     }
 
     async _performSanityChecks() {
@@ -189,13 +193,26 @@ const DesktopManager = class {
             await runDialog;
         }
 
-        const isFolder = this._desktopDir.query_file_type(
+        const fileType = this._desktopDir.query_file_type(
             Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
-            null) === Gio.FileType.DIRECTORY;
-        if (!isFolder) {
+            null);
+        if (fileType === Gio.FileType.SYMBOLIC_LINK) {
+            const header = _('Can Not Show the Desktop');
+            // TRANSLATORS: {desktop} folder path automatically inserted
+            const text = _('The Desktop folder {desktop} is a Symbolic Link\n\nPlease set the Desktop Folder to a real Folder');
             const errorDialog = this.showError(
-                _('Can Not Show the Desktop'),
-                _(`The Desktop folder ${this._desktopDir.get_path()} does not exist, or is not a Directory\n\nCheck your xdg-utils installation and set the correct Desktop Folder`)
+                header,
+                text.replace('{desktop}', this._desktopDir.get_path())
+            );
+            await errorDialog.run();
+            this._desktops.forEach(d => d.setErrorState());
+        } else if (fileType !== Gio.FileType.DIRECTORY) {
+            const header = _('Can Not Show the Desktop');
+            // TRANSLATORS: {desktop} folder path automatically inserted
+            const text = _('The Desktop folder {desktop} does not exist, or is not a Directory\n\nCheck your xdg-utils installation and set the correct Desktop Folder');
+            const errorDialog = this.showError(
+                header,
+                text.replace('{desktop}', this._desktopDir.get_path())
             );
             await errorDialog.run();
             this._desktops.forEach(d => d.setErrorState());
@@ -203,30 +220,25 @@ const DesktopManager = class {
 
         const inodeHandlers = Gio.AppInfo.get_all_for_type('inode/directory');
         if (!GLib.find_program_in_path('nautilus')) {
-            const errorDialog = this.showError(
-                _('GNOME Files not found'),
-                _('The GNOME Files application is required by Gtk4 Desktop Icons NG.')
-            );
+            const header = _('GNOME Files not found');
+            const text = _('The GNOME Files application is required by Gtk4 Desktop Icons NG.');
+            const errorDialog = this.showError(header, text);
             await errorDialog.run();
         }
 
         if (!inodeHandlers.length) {
             const helpURL = 'https://gitlab.com/smedius/desktop-icons-ng/-/issues/73';
-            const errorDialog = this.showError(
-                _('There is no default File Manager'),
-                _('There is no application that handles mimetype "inode/directory"'),
-                helpURL
-            );
+            const header = _('There is no default File Manager');
+            const text = _('There is no application that handles mimetype "inode/directory"');
+            const errorDialog = this.showError(header, text, helpURL);
             await errorDialog.run();
         }
 
         if (!inodeHandlers.map(a => a.get_id()).includes('org.gnome.Nautilus.desktop')) {
             const helpURL = 'https://gitlab.com/smedius/desktop-icons-ng/-/issues/73';
-            const errorDialog = this.showError(
-                _('Gnome Files is not registered as a File Manager'),
-                _('The Gnome Files application is not programmed to open Folders!\nCheck your xdg-utils installation\nCheck Gnome Files .desktop File installation'),
-                helpURL
-            );
+            const header = _('Gnome Files is not registered as a File Manager');
+            const text = _('The Gnome Files application is not programmed to open Folders!\nCheck your xdg-utils installation\nCheck Gnome Files .desktop File installation');
+            const errorDialog = this.showError(header, text, helpURL);
             await errorDialog.run();
         }
     }
@@ -261,18 +273,13 @@ const DesktopManager = class {
             this._displayList.forEach(f => f.onDestroy());
         }
 
+        this._stopWidgetDisplay();
         this.windowManager.destroyDesktops();
     }
 
     // Keyboard and Mouse Events
 
-    async onPressButton(X, Y,
-        x, y,
-        button,
-        shiftPressed,
-        controlPressed,
-        grid
-    ) {
+    onPressButton(X, Y, x, y, button, shiftPressed, controlPressed) {
         this._clickX = Math.floor(X);
         this._clickY = Math.floor(Y);
 
@@ -284,7 +291,9 @@ const DesktopManager = class {
             }
             this.dragManager.startRubberband(X, Y);
         }
+    }
 
+    async onReleaseButton(X, Y, x, y, button, isShift, isCtrl, grid) {
         // Right Click
         if (button === 3) {
             await this.desktopMenuManager
@@ -293,6 +302,28 @@ const DesktopManager = class {
         }
     }
 
+    onLongPressButton(_X, _Y, _x, _y, button, _isShift, _isCtrl, _grid) {
+        if (button === 3)
+            this.mainApp.activate_action('displayShellBackgroundMenu', null);
+    }
+    
+    onWidgetDisplayChanged() {
+        if (this.Prefs.showDesktopWidgets)
+            this._startWidgetDisplay();
+        else
+            this._stopWidgetDisplay();
+    }
+
+    _startWidgetDisplay() {
+        this.widgetManager
+        .startWidgetDisplay(this._desktops, {redisplay: true})
+        .catch(e => logError(e));
+    }
+
+    _stopWidgetDisplay() {
+        this.widgetManager.stopWidgetDisplay();
+    }
+
     onKeyPress(keyval, keycode, state, grid) {
         this.keyEventGrid = grid;
         if (this.popupmenu || this.fileItemMenu.popupmenu)
@@ -366,6 +397,27 @@ const DesktopManager = class {
         return false;
     }
 
+    clearAllLayersFromGrids() {
+        // Icons: clear from all grids
+        this._displayList.forEach(x => x.removeFromGrid());
+
+        // Widgets: clear from all grids
+        this.widgetManager.clearFromGrids();
+    }
+
+    async applyDesktopLayoutChange({redisplay, monitorschanged, gridschanged}) {
+        this._performSanityChecks();
+
+        // Icons first
+        await this.reFrameDesktop({
+            redisplay,
+            monitorschanged,
+            gridschanged,
+        });
+
+        await this.widgetManager.applyLayoutChange(this._desktops, {redisplay});
+    }
+
 
     // Destktop Icon Placement and Display
     // ********************************************************************** */
@@ -836,7 +888,7 @@ const DesktopManager = class {
          * @param {integer} b the secondfile size
          */
         function bySize(a, b) {
-            return  a.fileSize - b.fileSize;
+            return a.fileSize - b.fileSize;
         }
 
         /**
@@ -846,7 +898,7 @@ const DesktopManager = class {
          * @param {integer} b the second file timestamp
          */
         function byTime(a, b) {
-            return  a._modifiedTime - b._modifiedTime;
+            return a._modifiedTime - b._modifiedTime;
         }
 
         let specialFiles = [];
@@ -1049,7 +1101,7 @@ const DesktopManager = class {
     _sortByOriginalPosition() {
         let cornerInversion = this.Prefs.StartCorner;
         if (!cornerInversion[0] && !cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.X < b.X)
                     return -1;
                 if (a.X > b.X)
@@ -1062,7 +1114,7 @@ const DesktopManager = class {
             });
         }
         if (cornerInversion[0] && cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.X < b.X)
                     return 1;
                 if (a.X > b.X)
@@ -1075,7 +1127,7 @@ const DesktopManager = class {
             });
         }
         if (cornerInversion[0] && !cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.X < b.X)
                     return 1;
                 if (a.X > b.X)
@@ -1088,7 +1140,7 @@ const DesktopManager = class {
             });
         }
         if (!cornerInversion[0] && cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.X < b.X)
                     return -1;
                 if (a.X > b.X)
@@ -1105,7 +1157,7 @@ const DesktopManager = class {
     _sortByCurrentPosition() {
         let cornerInversion = this.Prefs.StartCorner;
         if (!cornerInversion[0] && !cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.x < b.x)
                     return -1;
                 if (a.x > b.x)
@@ -1118,7 +1170,7 @@ const DesktopManager = class {
             });
         }
         if (cornerInversion[0] && cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.x < b.x)
                     return 1;
                 if (a.x > b.x)
@@ -1131,7 +1183,7 @@ const DesktopManager = class {
             });
         }
         if (cornerInversion[0] && !cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.x < b.x)
                     return 1;
                 if (a.x > b.x)
@@ -1144,7 +1196,7 @@ const DesktopManager = class {
             });
         }
         if (!cornerInversion[0] && cornerInversion[1]) {
-            this._displayList.sort((a, b) =>   {
+            this._displayList.sort((a, b) => {
                 if (a.x < b.x)
                     return -1;
                 if (a.x > b.x)
@@ -1287,6 +1339,7 @@ const DesktopManager = class {
                 this.unselectAll();
 
             this.mainApp.activate_action('textEntryAccelsTurnOn', null);
+            this._displayList.forEach(f => (f.opacity = 1));
             this._findFileWindow.destroy();
             this._findFileWindow = null;
         });
@@ -1314,10 +1367,18 @@ const DesktopManager = class {
         if (found.length !== 0) {
             if (setselected) {
                 this.unselectAll();
-                found.map(f => f.setSelected());
+                const notfound = this._displayList.filter(
+                    f => !found.includes(f)
+                );
+                found.forEach(f => {
+                    f.setSelected();
+                    f.opacity = 1;
+                });
+                notfound.forEach(f => (f.opacity = 0.2));
             }
             return true;
         } else {
+            this.unselectAll();
             return false;
         }
     }
@@ -1337,7 +1398,7 @@ const DesktopManager = class {
          * @param {integer} b fileItem file modified time
          */
         function byTime(a, b) {
-            return  a._modifiedTime - b._modifiedTime;
+            return a._modifiedTime - b._modifiedTime;
         }
         this._displayList.sort(byTime);
         this._reassignFilesToDesktop();
@@ -1349,7 +1410,7 @@ const DesktopManager = class {
          * @param {integer} b fileItem fileSize
          */
         function bySize(a, b) {
-            return  a.fileSize - b.fileSize;
+            return a.fileSize - b.fileSize;
         }
         this._displayList.sort(bySize);
         this._reassignFilesToDesktop();
@@ -1450,7 +1511,10 @@ const DesktopManager = class {
     }
 
     unselectAll() {
-        this._displayList.map(f => f.unsetSelected());
+        this._displayList.forEach(f => {
+            f.unsetSelected();
+            f.opacity = 1;
+        });
         this.fileItemMenu.activeFileItem = null;
     }
 
@@ -1555,7 +1619,7 @@ const DesktopManager = class {
             position = [this._clickX, this._clickY];
 
 
-        const baseName = suggestedName ? suggestedName :  _('New Folder');
+        const baseName = suggestedName ? suggestedName : _('New Folder');
         let newName = this.desktopMonitor.getDesktopUniqueFileName(baseName);
 
         if (newName) {
@@ -1701,6 +1765,14 @@ const DesktopManager = class {
         });
     }
 
+    onDarkModeChanged() {
+        this.widgetManager.onThemeChanged();
+    }
+
+    onAnimationChanged() {
+        this.widgetManager.onAnimationChanged();
+    }
+
     // Getters and Setters
 
     get _desktopDir() {
