@@ -17,6 +17,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 import {Adw, GLib, Gtk, Gio, Gdk, DesktopAppInfo} from '../dependencies/gi.js';
+import {DesktopWidgetCapability} from '../dependencies/gi.js';
 import {_} from '../dependencies/gettext.js';
 
 export {Preferences};
@@ -31,6 +32,10 @@ const Preferences = class {
         this._Enums = Data.Enums;
         let schemaSource = GioSSS.get_default();
         this._desktopManager = null;
+        this._widgetState = null;
+        this._widgetStateMonitor = null;
+        this._suppressWidgetMonitorEvent = false;
+        this.desktopWidgetCapability = DesktopWidgetCapability;
 
         // Adw Style Manager
         this._adwStyleManager =
@@ -208,6 +213,9 @@ const Preferences = class {
         this.freePositionIcons =
             this.desktopSettings.get_boolean('free-position-icons');
 
+        this.desktopWidgetsEnabled =
+            this.desktopSettings.get_boolean('show-desktop-widgets');
+
         this.CLICK_POLICY_SINGLE =
             this.nautilusSettings.get_string('click-policy') === 'single';
 
@@ -220,6 +228,11 @@ const Preferences = class {
 
         this.darkmode = this._adwStyleManager.get_dark();
         this._premultiplied = this._getPreMultiplied();
+
+        // Gtk Theme and animation Changes
+        this._gtkSettings =
+            Gtk.Settings.get_for_display(Gdk.Display.get_default());
+        this._refreshAnimations();
     }
 
     getAdwPreferencesWindow() {
@@ -241,14 +254,68 @@ const Preferences = class {
     init(desktopManager) {
         this._desktopManager = desktopManager;
         this._desktopIconsUtil = desktopManager.DesktopIconsUtil;
+
         this._configureSelectionColor();
         this._configureHoverColor();
         this._setCSSColors();
         this._initLocalCSSprovider();
         this._monitorDesktopSettings();
         this._monitorTerminalSettings();
+        this._monitorWidgetState();
     }
 
+    _monitorWidgetState() {
+        if (!this._desktopIconsUtil)
+            return;
+
+        const widgetsFile = this._desktopIconsUtil.getWidgetsStateFile();
+        if (!widgetsFile)
+            return;
+
+        try {
+            this._widgetStateMonitor =
+                widgetsFile.monitor_file(
+                    Gio.FileMonitorFlags.WATCH_MOVES,
+                    null
+                );
+
+            this._widgetStateMonitor.set_rate_limit(500);
+
+            this._widgetStateMonitor.connect('changed', () => {
+                if (this._suppressWidgetMonitorEvent)
+                    return;
+
+                this._loadWidgetState()
+                .catch(e => {
+                    console.log(
+                        'Error loading widget state from widgets.json:',
+                        e.message ?? e
+                    );
+                    this._widgetState = null;
+                    this._applyWidgetStateToManager();
+                });
+            });
+        } catch (e) {
+            console.log(
+                'Error monitoring widget state from widgets.json:',
+                e.message ?? e
+            );
+            this._widgetStateMonitor = null;
+            return;
+        }
+
+        this._loadWidgetState()
+        .catch(e => {
+            console.log(
+                'Error loading widget state from widgets.json:',
+                e.message ?? e
+            );
+            this._widgetState = null;
+            this._applyWidgetStateToManager();
+        });
+    }
+
+
     _monitorDesktopSettings() {
         if (!this._desktopManager)
             return;
@@ -388,6 +455,12 @@ const Preferences = class {
                     this.desktopSettings.get_boolean('free-position-icons');
             }
 
+            if (key === 'show-desktop-widgets') {
+                this.desktopWidgetsEnabled =
+                    this.desktopSettings.get_boolean('show-desktop-widgets');
+                this._desktopManager.onWidgetDisplayChanged();
+            }
+
             // fallthrough
             this._desktopManager.reLoadDesktop()
             .catch(e => {
@@ -449,12 +522,11 @@ const Preferences = class {
             this._refreshDesktopAndColors();
         });
 
-        // Gtk Theme Changes
-        this._gtkSettings =
-            Gtk.Settings.get_for_display(Gdk.Display.get_default());
-
-        this._gtkSettings.connect('notify::gtk-theme-name', () => {
-            this._refreshDesktopAndColors();
+        this._gtkSettings.connect('notify', (actor, paramSpec) => {
+            if (paramSpec.name === 'gtk-theme-name')
+                this._refreshDesktopAndColors();
+            if (paramSpec.name === 'gtk-enable-animations')
+                this._refreshAnimations();
         });
 
         // Callback to handle accent color changes
@@ -593,6 +665,7 @@ const Preferences = class {
 
         this._configureSelectionColor();
         this._setCSSColors();
+        this._desktopManager?.onDarkModeChanged();
     }
 
     _refreshDesktopAndColors() {
@@ -607,17 +680,65 @@ const Preferences = class {
         });
     }
 
+    _refreshAnimations() {
+        this.globalAnimations =
+            this._gtkSettings.gtk_enable_animations ?? false;
+        const enabled = this.globalAnimations ? 'enabled' : 'disabled';
+        console.log('System animations are', enabled);
+        this._desktopManager?.onAnimationChanged();
+    }
+
     _initLocalCSSprovider() {
         const cssProvider = new Gtk.CssProvider();
         const resourcePath = this._mainApp.get_resource_base_path();
 
         cssProvider.load_from_resource(`${resourcePath}/stylesheet.css`);
 
+        // Load user-local stylesheet override from config directory
+        this._cssOverrideProvider = new Gtk.CssProvider();
+        this._loadCSSOverride();
+
         Gtk.StyleContext.add_provider_for_display(
             Gdk.Display.get_default(),
             cssProvider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        );
+        Gtk.StyleContext.add_provider_for_display(
+            Gdk.Display.get_default(),
+            this._cssOverrideProvider,
             Gtk.STYLE_PROVIDER_PRIORITY_USER
         );
+        
+        this._createReloadCSSAction();
+    }
+    
+    _loadCSSOverride() {
+        const configDir = GLib.get_user_config_dir();
+        const cssOverridePath = GLib.build_filenamev([
+            configDir,
+            this._mainApp.get_application_id(),
+            'stylesheet-override.css'
+        ]);
+        const overrideFile = Gio.File.new_for_path(cssOverridePath);
+        const overrideExists = overrideFile.query_exists(null);
+
+        if (!overrideExists) {
+            this._cssOverrideProvider.load_from_string('');
+            return;
+        }
+
+        try {
+            this._cssOverrideProvider.load_from_path(cssOverridePath);
+            console.info('Loaded CSS override file');
+        } catch (e) {
+            console.error(`Failed to load CSS override: ${e}`);
+        }
+    }
+
+    _createReloadCSSAction() {
+        const reloadCSSAction = Gio.SimpleAction.new('reloadCSS', null);
+        reloadCSSAction.connect('activate', this._loadCSSOverride.bind(this));
+        this._mainApp.add_action(reloadCSSAction);
     }
 
     _setupTerminalMonitors() {
@@ -824,6 +945,70 @@ const Preferences = class {
         }
     }
 
+    /*
+     * Load widget layout/config state from widgets.json into _widgetState.
+     */
+    async _loadWidgetState(cancellable = null) {
+        if (!this._desktopIconsUtil)
+            return;
+
+        const file = this._desktopIconsUtil.getWidgetsStateFile();
+        this._widgetState =
+            await this._desktopIconsUtil.readJsonFile(file, cancellable)
+            .catch(e => {
+                console.log(
+                    'Error reading widget state from widgets.json:',
+                    e.message ?? e
+                );
+                return null;
+            });
+
+        this._applyWidgetStateToManager();
+    }
+
+    _applyWidgetStateToManager() {
+        if (!this._desktopManager?.widgetManager)
+            return;
+
+        const state =
+            this._widgetState ??
+            {
+                version: 1,
+                instances: [],
+            };
+
+        this._desktopManager.widgetManager.loadState(state);
+    }
+
+    /*
+     * Save widget layout/config state back to widgets.json.
+     *
+     * @param {object|null} state plain JSON-serializable object
+     *                            (or null to just clear cache)
+     */
+    async _saveWidgetState(state, cancellable = null) {
+        if (!this._desktopIconsUtil)
+            return;
+
+        this._widgetState = state ?? null;
+
+        if (!state)
+            return;
+
+        const file = this._desktopIconsUtil.getWidgetsStateFile();
+
+        // Prevent triggering monitor event while we write the file
+        this._suppressWidgetMonitorEvent = true;
+
+        await this._desktopIconsUtil.writeJsonFile(file, state, cancellable);
+
+        // Allow monitor events after idle
+        GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
+            this._suppressWidgetMonitorEvent = false;
+            return GLib.SOURCE_REMOVE;
+        });
+    }
+
     // Setters
     /**
      * @param {any} order
@@ -842,6 +1027,16 @@ const Preferences = class {
         this._setPreMultiplied(boolean);
     }
 
+    set widgetState(state = null) {
+        this._widgetState = state;
+        this._saveWidgetState(state).catch(e => {
+            console.log(
+                'Error saving widget state to widgets.json:',
+                e.message ?? e
+            );
+        });
+    }
+
     // Getters
     get StartCorner() {
         // Return a shallow copy that can be mutated without affecting other
@@ -887,4 +1082,12 @@ const Preferences = class {
             return false;
         return this._premultiplied;
     }
+
+    get widgetState() {
+        return this._widgetState;
+    }
+
+    get showDesktopWidgets() {
+        return this.desktopWidgetsEnabled && this.desktopWidgetCapability;
+    }
 };
